#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "tcron.h"
#include "util.h"

int qcmp_cron_t(const void *a, const void *b)
{
  return *((cron_t *)a)- *((cron_t *)b);
}

void sort_cron_t(cron_t *a, int an)
{
  qsort(a,an,sizeof(cron_t),qcmp_cron_t);
}

int delDup(cron_t *a, int an)
{
  int i,c=0;
  if (an<2)
    return an;
  sort_cron_t(a,an);
  for(i=0;i<an;i++)
    if (!i || a[i]!=a[i-1])
      a[c++]=a[i];
  return c;
}


int get_cron_t(char **ss, int vmin, int vmax)
{
  char *s, *t;

  s=t=*ss;

  if (*t=='*') {
    *ss=t+1;
    return STAR;
  }

  while (*t && isdigit(*t))
    t++;

  int n=t-s;

  if (n==0)
    p_err("-->%s should be a integer\n", s);

  if (n>4)
    p_err("%s number too long ...\n",s);

  char *ts=malloc(n+1);
  memcpy(ts,s,n);
  ts[n]=0;

  int val=atoi(ts);
  free(ts);

  if (val < vmin || val > vmax)
    p_err("-->%s should be between %d %d\n",s, vmin,vmax);

  *ss=t;
  return val;
}


int get_int_list(char **ss, cron_t **intarr, int vmin, int vmax)
{
  char *s=*ss;
  cron_t *iarr=NULL;
  int iarrN=0;

  while (!is_spc(*s)) {
    int v;

    if (*s==',') {
      s=skip_spc(s+1);
    }

    v=get_cron_t(&s,vmin,vmax);
    iarr=realloc(iarr,sizeof(int)*(iarrN+1));
    iarr[iarrN++]=v;
    s=skip_spc(s);
    if (*s=='-') {
      s=skip_spc(s+1);
      int v2=get_cron_t(&s,vmin,vmax);
      int i,step=1;

      s=skip_spc(s);

      if (*s=='/') {
        s=skip_spc(s+1);
        step=get_cron_t(&s,1,vmax);
      }

      if (v>v2)
        swapint(&v,&v2);

      iarr=realloc(iarr,sizeof(cron_t)*(iarrN+v2-v+1));

      for(i=v;i<=v2;i+=step)
        iarr[iarrN++]=i;

    }

    s=skip_spc(s);
    if (*s!=',')
      break;
  }


  iarrN=delDup(iarr,iarrN);

  int i;

  for(i=0;i<iarrN;i++)
    if (iarr[i]==STAR) {
       iarr[i]=STAR;
       iarrN=1;
    }


  *ss=s;

  *intarr=iarr;
  return iarrN;
}

char *to_spc_colon(char *p);

void tcron_parse_line(char *line_str,CRON_ENT *pcent, int b_apId, int b_month0)
{
   int atjob=0;
   int i;
   int len=strlen(line_str);

   mbzero(pcent,sizeof(CRON_ENT));

   if (len && line_str[len-1]=='\n')
     line_str[len-1]=0;

   if (!len)
     return;

   char *p=line_str;

   p=skip_spc(p);
   if (*p=='#' || *p==0)
     return;

   if (*p=='!') {  // application type escape char
     char *q;
     p++;
     p=skip_spc(p);
     q=to_spc_colon(p);

     if (b_apId)
       pcent->apId=mstrndup(p,q-p);

     p=skip_spc(q);

     if (*p==':') {
       p++;
       p=skip_spc(p);
       q=to_spc(p);


       if (b_apId)
         pcent->apData=mstrndup(p,q-p);

       p=skip_spc(q);
     }
   }


   if (*p=='@') {
     atjob=1;
     p++;
   }

   cron_t *minArr;
   int minArrN;
   p=skip_spc(p);
   minArrN=get_int_list(&p,&minArr,0,59);
   pcent->minArrN=minArrN;
   pcent->minArr=minArr;
#if     DUMP
     dbg("\nmin -> ");
     for(i=0;i<minArrN;i++)
       dbg("%d ",minArr[i]);
#endif

   cron_t *hourArr;
   int hourArrN;
   p=skip_spc(p);
   hourArrN=get_int_list(&p,&hourArr,0,23);
   pcent->hourArrN=hourArrN;
   pcent->hourArr=hourArr;
#if     DUMP
     dbg("\nhours -> ");
     for(i=0;i<hourArrN;i++)
       dbg("%d ",hourArr[i]);
#endif
   p=skip_spc(p);
   cron_t *daymonArr;
   int daymonArrN;
   daymonArrN=get_int_list(&p,&daymonArr,1,31);
   pcent->daymonArrN=daymonArrN;
   pcent->daymonArr=daymonArr;

#if     DUMP
     dbg("\ndaymon -> ");
     for(i=0;i<daymonArrN;i++)
       dbg("%d ",daymonArr[i]);
#endif
   p=skip_spc(p);
   cron_t *monArr;
   int monArrN;
   monArrN=get_int_list(&p,&monArr,1,12);

   pcent->monArrN=monArrN;
   pcent->monArr=monArr;

   if (b_month0) {
     for(i=0;i<monArrN;i++)
       if (monArr[i]!=STAR)
         monArr[i]--;
   }

#if     DUMP
     dbg("\nmonth -> ");
     for(i=0;i<monArrN;i++)
       dbg("%d ",monArr[i]);
#endif

   p=skip_spc(p);

   if (atjob) {
     cron_t *yearArr;
     int yearArrN;

     yearArrN=get_int_list(&p,&yearArr,2000,32767);
     if (yearArrN!=1)
       p_err("# of atjob year should be one");
     pcent->dayweekArrN=yearArrN;
     pcent->dayweekArr=yearArr;
   } else {
     cron_t *dayweekArr;
     int dayweekArrN;
     dayweekArrN=get_int_list(&p,&dayweekArr,0,7);

     for(i=0;i<dayweekArrN;i++)
       if (dayweekArr[i]==7)
         dayweekArr[i]=0;

     pcent->dayweekArrN=dayweekArrN;
     pcent->dayweekArr=dayweekArr;
#if     DUMP
       dbg("\ndayweek -> ");
       for(i=0;i<dayweekArrN;i++)
         dbg("%d ",dayweekArr[i]);
#endif
   }

   p=skip_spc(p);

   pcent->cmd=strdup(p);
#if     DUMP
     dbg("\ncmd -> %s\n",p);
#endif
}



int tcron_parse_file_fp(FILE *fp, CRON_ENT **r_centArr, char *apId, int b_month0)
{
   CRON_ENT *centArr=NULL;
   int centArrN=0;

   while (!feof(fp)) {
     CRON_ENT cr_ent;
     char buf[1024];

     buf[0]=0;
     fgets(buf,sizeof(buf),fp);

     tcron_parse_line(buf,&cr_ent, apId?1:0, b_month0);

     if (!cr_ent.minArrN)
       continue;

     // if apId is set, only matched entries will be read, except apId is '*',
     if (apId && apId[0]!='*' && (!cr_ent.apId || strcmp(cr_ent.apId,apId)) ) {
       tcron_free_CRON_ENT(&cr_ent);
       continue;
     }

     centArr=(CRON_ENT *)realloc(centArr,sizeof(CRON_ENT)*(centArrN+1));
     centArr[centArrN++]=cr_ent;
   }

   fclose(fp);

   if (!centArrN && centArr) {
     free(centArr); centArr=NULL;
   }

   *r_centArr=centArr;

//   dbg("centArrN %d\n",centArrN);
   return centArrN;
}

int tcron_parse_file(char *fname, CRON_ENT **r_centArr, char *apId, int b_month0)
{
   FILE *fp;

   if ((fp=fopen(fname,"r"))==NULL)
     p_err("file %s was not found ??");

   return tcron_parse_file_fp(fp, r_centArr, apId, b_month0);
}

void tcron_free_CRON_ENT(CRON_ENT *pcrent)
{
  free(pcrent->minArr);
  free(pcrent->hourArr);
  free(pcrent->daymonArr);
  free(pcrent->monArr);
  free(pcrent->dayweekArr);
  if (pcrent->apId)
    free(pcrent->apId);
}

