#include <stdio.h>
#include <stdarg.h>
#include <pwd.h>
#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "tcron.h"
#include "util.h"

#if 0
static int caller_identity;

void set_caller_identity(int ident)
{
  caller_identity=ident;
}
#endif

static FILE *logfp;

void p_err(char *fmt,...)
{
  va_list args;

  va_start(args, fmt);
  vfprintf(stderr, fmt, args);
  fflush(stderr);
  va_end(args);
  exit(-1);
}

void init_log()
{
  int fd;

  if ((logfp=fopen("/var/log/tcrond.log","a"))==NULL) {
    exit(-1);
  }

  fd=fileno(logfp);
  dup2(fd,1);
  dup2(fd,2);
  setbuf(logfp,NULL);
}

void close_log()
{
  fclose(logfp);
}

void dbg(char *fmt,...)
{
  va_list args;

  if (!logfp)
    logfp=stdout;
  va_start(args, fmt);
  vfprintf(logfp,fmt, args);
  va_end(args);
  fflush(stdout);
}



char *skip_spc(char *s)
{
  while (*s==' '||*s==9) s++;
  return s;
}

char *to_spc(char *s)
{
  while (*s!=' ' && *s!=9 && *s) s++;
  return s;
}

char *to_spc_colon(char *s)
{
  while (*s!=' ' && *s!=9 && *s!=':' && *s) s++;
  return s;
}

int is_spc(char c)
{
  return c==' '||c==9;
}

void swapint(int *a, int *b)
{
  int t;
  t=*a;
  *a=*b;
  *b=t;
}

char *genTmpName(char *username)
{
  int pid=getpid();
  int count;

  for(count=0;;count++) {
    struct stat st;
    char fname[512];

    sprintf(fname,"/tmp/tcron.%s.%d.%d",username,pid,count);

    if (stat(fname,&st)<0) {  // test to avoid symlink attack
      return strdup(fname);
    } else
      unlink(fname);
  }
}

char *read_config(char *confstr)
{
  FILE *fp;
  int conflen=strlen(confstr);
  static char sstr[80];
  char *rstr=NULL;

  if ((fp=fopen("/etc/tcron.conf","r"))==NULL)
    return NULL;

  while (!feof(fp)) {
    char line[80];
    int linen;
    int val;
    char *p;
    int klen;

    line[0]=0;
    fgets(line,sizeof(line),fp);
    linen=strlen(line);

    if (linen <= conflen)
      continue;

    if (line[linen-1]=='\n')
      line[linen-1]=0;

    p=to_spc(line);

    klen = p-line;
    if (klen != conflen)
      continue;

    if (!memcmp(line,confstr,klen)) {
      char *q=skip_spc(p);
      strcpy(sstr,q);
      rstr=sstr;
      break;
    }
  }

  fclose(fp);
  return rstr;
}

char *mstrndup(char *s, int n)
{
  int len=strlen(s);
  char *t;

  if (len <= n)
    return strdup(s);

  t=malloc(n+1);
  memcpy(t,s,n);
  t[n]=0;

  return t;
}
